/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import {Button} from '@ladifire-ui-react/button';
import {classNames, SlotProvider, unwrapDOMRef, useDOMRef, useHasChild, useStyleProps} from '@ladifire-ui-react/utils';
import CrossLarge from '@spectrum-icons/ui/CrossLarge';
import {DialogContext, DialogContextValue} from './context';
import {DismissButton} from '@react-aria/overlays';
import {DOMRef} from '@react-ui-types/shared';
import {FocusScope} from '@react-aria/focus';
import {Grid} from '@ladifire-ui-react/layout';
import {mergeProps} from '@react-aria/utils';
import * as React from 'react';
import {SpectrumDialogProps} from '@react-ui-types/dialog';
import styles from '@adobe/spectrum-css-temp/components/dialog/vars.css';
import {useDialog} from '@react-aria/dialog';

import gs from '@ladifire-css/sass/index.scss';
import ds from '@ladifire-css/sass/components/dialog.scss';

let sizeMap = {
  S: 'small',
  M: 'medium',
  L: 'large',
  fullscreen: 'fullscreen',
  fullscreenTakeover: 'fullscreenTakeover'
};

function Dialog(props: SpectrumDialogProps, ref: DOMRef) {
  let {
    type = 'modal',
    ...contextProps
  } = React.useContext(DialogContext) || {} as DialogContextValue;
  let {
    children,
    isDismissable = contextProps.isDismissable,
    onDismiss = contextProps.onClose,
    size,
    ...otherProps
  } = props;
  let {styleProps} = useStyleProps(otherProps);

  size = type === 'popover' ? (size || 'S') : (size || 'L');

  let domRef = useDOMRef(ref);
  let gridRef = React.useRef();
  let sizeVariant = sizeMap[type] || sizeMap[size];
  let {dialogProps, titleProps} = useDialog(mergeProps(contextProps, props), domRef);

  let hasHeader = useHasChild(`.${styles['spectrum-Dialog-header']}`, unwrapDOMRef(gridRef));
  let hasFooter = useHasChild(`.${styles['spectrum-Dialog-footer']}`, unwrapDOMRef(gridRef));

  let slots = {
    hero: {UNSAFE_className: ds.dialog__hero},
    header: {UNSAFE_className: ds.dialog__header},
    heading: {UNSAFE_className: classNames(ds, ds.dialog__heading, {[ds.dialog__heading_no__header]: !hasHeader}), ...titleProps},
    typeIcon: {UNSAFE_className: ds.dialog__type_icon},
    divider: {UNSAFE_className: ds.dialog__divider, size: 'M'},
    content: {UNSAFE_className: ds.dialog__content},
    footer: {UNSAFE_className: ds.dialog__footer},
    buttonGroup: {UNSAFE_className: classNames(ds, ds.dialog__buttonGroup, {[ds.dialog__buttonGroup_no_footer]: !hasFooter}), align: 'end'}
  };

  // If rendered in a popover or tray there won't be a visible dismiss button,
  // so we render a hidden one for screen readers.
  let dismissButton: JSX.Element;
  if (type === 'popover' || type === 'tray') {
    dismissButton = <DismissButton onDismiss={onDismiss} />;
  }

  return (
    <FocusScope contain restoreFocus>
      <section
        {...styleProps}
        {...dialogProps}
        className={classNames(
          gs,
          gs.display__flex,
          gs.box_sizing__border_box,
          gs.width__fit_content,
          gs.max_width__100_percent,
          gs.max_height__inherit,
          gs.outline__none,
          ds.dialog__min_width,
          classNames(
            styles,
            'spectrum-Dialog',
            {
              [`spectrum-Dialog--${sizeVariant}`]: sizeVariant,
              'spectrum-Dialog--dismissable': isDismissable
            },
            styleProps.className
          )
        )}
        // className={classNames(
        //   styles,
        //   'spectrum-Dialog',
        //   {
        //     [`spectrum-Dialog--${sizeVariant}`]: sizeVariant,
        //     'spectrum-Dialog--dismissable': isDismissable
        //   },
        //   styleProps.className
        // )}
        ref={domRef}
      >
        <Grid ref={gridRef} UNSAFE_className={classNames(
          ds,
          ds.dialog__grid, {
            [ds.dialog__dismissable]: isDismissable,
          }
        )}>
          <SlotProvider slots={slots}>
            {children}
          </SlotProvider>
          {isDismissable &&
          <Button
            variant='transparent'
            square
            rounded
            className={ds.dialog__closeButton}
            aria-label='dismiss'
            onPress={onDismiss}
          >
            <CrossLarge size="L" />
          </Button>
          }
        </Grid>
        {dismissButton}
      </section>
    </FocusScope>
  );
}

/**
 * Dialogs are windows containing contextual information, tasks, or workflows that appear over the user interface.
 * Depending on the kind of Dialog, further interactions may be blocked until the Dialog is acknowledged.
 */
let _Dialog = React.forwardRef(Dialog);
export {_Dialog as Dialog};
